<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

//Get saved values
$saved_values = get_option('wc_szamlazz_vat_overrides');
$countries_obj = new WC_Countries();
$countries = $countries_obj->__get('countries');

//Set valid documents for automation
$line_item_types = array(
	'product' => __('Line item(product)', 'wc-szamlazz'),
	'shipping' => __('Shipping', 'wc-szamlazz'),
	'discount' => __('Discount', 'wc-szamlazz'),
	'fee' => __('Fee', 'wc-szamlazz'),
	'refund' => __('Refund', 'wc-szamlazz')
);

//When to generate these documents
$vat_types = WC_Szamlazz_Helpers::get_vat_types();

//Setup conditions
$conditions = array(
	'payment_method' => array(
		"label" => __('Payment method', 'wc-szamlazz'),
		'options' => $this->get_payment_methods()
	),
	'shipping_method' => array(
		"label" => __('Shipping method', 'wc-szamlazz'),
		'options' => $this->get_shipping_methods()
	),
	'type' => array(
		"label" => __('Order type', 'wc-szamlazz'),
		'options' => array(
			'individual' => __('Individual', 'wc-szamlazz'),
			'company' => __('Company', 'wc-szamlazz'),
		)
	),
	'product_category' => array(
		'label' => __('Product category', 'wc-szamlazz'),
		'options' => array()
	),
	'billing_address' => array(
		"label" => __('Billing address', 'wc-szamlazz'),
		'options' => array(
			'eu' => __('Inside the EU', 'wc-szamlazz'),
			'world' => __('Outside of the EU', 'wc-szamlazz'),
		)
	),
	'billing_country' => array(
		"label" => __('Billing country', 'wc-szamlazz'),
		'options' => $countries
	),
);

//Add category options
foreach (get_terms(array('taxonomy' => 'product_cat')) as $category) {
	$conditions['product_category']['options'][$category->term_id] = $category->name;
}

//Apply filters
$conditions = apply_filters('wc_szamlazz_notes_conditions', $conditions);

?>

<tr valign="top">
	<th scope="row" class="titledesc"><?php echo esc_html( $data['title'] ); ?></th>
	<td class="forminp <?php echo esc_attr( $data['class'] ); ?>">
		<div class="wc-szamlazz-settings-vat-overrides">
			<?php if($saved_values): ?>
				<?php foreach ( $saved_values as $automation_id => $automation ): ?>

					<div class="wc-szamlazz-settings-vat-override">
						<div class="wc-szamlazz-settings-vat-override-title">
							<span class="text"><?php esc_html_e('Set', 'wc-szamlazz'); ?></span>
							<div class="select-field">
								<label><span>-</span></label>
								<select class="wc-szamlazz-settings-vat-override-line-item" data-name="wc_szamlazz_vat_overrides[X][line_item]">
									<?php foreach ($line_item_types as $value => $label): ?>
										<option value="<?php echo esc_attr($value); ?>" <?php if(isset($automation['line_item'])) selected( $automation['line_item'], $value ); ?>><?php echo esc_html($label); ?></option>
									<?php endforeach; ?>
								</select>
							</div>
							<span class="text"><?php esc_html_e('vat type to', 'wc-szamlazz'); ?></span>
							<div class="select-field">
								<label><span>-</span></label>
								<select class="wc-szamlazz-settings-vat-override-vat-type" data-name="wc_szamlazz_vat_overrides[X][vat_type]">
									<?php foreach ($vat_types as $value => $label): ?>
										<option value="<?php echo esc_attr($value); ?>" <?php if(isset($automation['vat_type'])) selected( $automation['vat_type'], $value ); ?>><?php echo esc_html($label); ?></option>
									<?php endforeach; ?>
								</select>
							</div>
							<a href="#" class="delete-vat-override"><?php _e('delete', 'wc-szamlazz'); ?></a>
						</div>
						<div class="wc-szamlazz-settings-vat-override-if">
							<label>
								<input type="checkbox" data-name="wc_szamlazz_vat_overrides[X][condition_enabled]" <?php checked( $automation['conditional'] ); ?> class="condition" value="yes">
								<span><?php _e('Based on the following conditions:', 'wc-szamlazz'); ?></span>
							</label>
							<ul class="wc-szamlazz-settings-vat-override-if-options" <?php if(!$automation['conditional']): ?>style="display:none"<?php endif; ?> <?php if(isset($automation['conditions'])): ?>data-options="<?php echo esc_attr(json_encode($automation['conditions'])); ?>"<?php endif; ?>></ul>
						</div>
					</div>
				<?php endforeach; ?>
			<?php endif; ?>
		</div>
		<div class="wc-szamlazz-settings-vat-override-add">
			<a href="#"><span class="dashicons dashicons-plus-alt"></span> <span><?php _e('Add new override', 'wc-szamlazz'); ?></span></a>
		</div>
		<?php echo $this->get_description_html( $data ); // WPCS: XSS ok. ?>
	</td>
</tr>

<script type="text/html" id="wc_szamlazz_vat_override_sample_row">
	<div class="wc-szamlazz-settings-vat-override">
		<div class="wc-szamlazz-settings-vat-override-title">
			<span class="text"><?php esc_html_e('Set', 'wc-szamlazz'); ?></span>
			<div class="select-field">
				<label><span>-</span></label>
				<select class="wc-szamlazz-settings-vat-override-line-item" data-name="wc_szamlazz_vat_overrides[X][line_item]">
					<?php foreach ($line_item_types as $value => $label): ?>
						<option value="<?php echo esc_attr($value); ?>"><?php echo esc_html($label); ?></option>
					<?php endforeach; ?>
				</select>
			</div>
			<span class="text"><?php esc_html_e('vat type to', 'wc-szamlazz'); ?></span>
			<div class="select-field">
				<label><span>-</span></label>
				<select class="wc-szamlazz-settings-vat-override-vat-type" data-name="wc_szamlazz_vat_overrides[X][vat_type]">
					<?php foreach ($vat_types as $value => $label): ?>
						<option value="<?php echo esc_attr($value); ?>"><?php echo esc_html($label); ?></option>
					<?php endforeach; ?>
				</select>
			</div>
			<a href="#" class="delete-vat-override"><?php _e('delete', 'wc-szamlazz'); ?></a>
		</div>
		<div class="wc-szamlazz-settings-vat-override-if">
			<label>
				<input type="checkbox" data-name="wc_szamlazz_vat_overrides[X][condition_enabled]" class="condition" value="yes">
				<span><?php _e('Based on the following conditions:', 'wc-szamlazz'); ?></span>
			</label>
			<ul class="wc-szamlazz-settings-vat-override-if-options" style="display:none"></ul>
		</div>
	</div>
</script>

<script type="text/html" id="wc_szamlazz_vat_override_condition_sample_row">
	<li>
		<select class="condition" data-name="wc_szamlazz_vat_overrides[X][conditions][Y][category]">
			<?php foreach ($conditions as $condition_id => $condition): ?>
				<option value="<?php echo esc_attr($condition_id); ?>"><?php echo esc_html($condition['label']); ?></option>
			<?php endforeach; ?>
		</select>
		<select class="comparison" data-name="wc_szamlazz_vat_overrides[X][conditions][Y][comparison]">
			<option value="equal"><?php _e('Equal', 'wc-szamlazz'); ?></option>
			<option value="not_equal"><?php _e('Not equal', 'wc-szamlazz'); ?></option>
		</select>
		<?php foreach ($conditions as $condition_id => $condition): ?>
			<select class="value <?php if($condition_id == 'payment_method'): ?>selected<?php endif; ?>" data-condition="<?php echo esc_attr($condition_id); ?>" data-name="wc_szamlazz_vat_overrides[X][conditions][Y][<?php echo esc_attr($condition_id); ?>]" <?php if($condition_id != 'payment_method'): ?>disabled="disabled"<?php endif; ?>>
				<?php foreach ($condition['options'] as $option_id => $option_name): ?>
					<option value="<?php echo esc_attr($option_id); ?>"><?php echo esc_html($option_name); ?></option>
				<?php endforeach; ?>
			</select>
		<?php endforeach; ?>
		<a href="#" class="add-row"><span class="dashicons dashicons-plus-alt"></span></a>
		<a href="#" class="delete-row"><span class="dashicons dashicons-dismiss"></span></a>
	</li>
</script>
